// ./func_handlerequest.go.gen requests.txt
// MACHINE GENERATED BY THE COMMAND ABOVE; DO NOT EDIT

package nslcd_server

import (
	"fmt"
	"io"
	"os"
	"time"

	p "git.lukeshu.com/go/libnslcd/nslcd_proto"
	"golang.org/x/sys/unix"
)

const sensitive = "<omitted-from-log>"

func maybePanic(err error) {
	if err != nil {
		panic(err)
	}
}

type Limits struct {
	// What is the maximum total amount of time that we spend
	// handling a single request.  This includes both the time
	// reading the request and the time creating and writing the
	// response.
	Timeout time.Duration

	// How long can we spend reading a request?
	ReadTimeout time.Duration

	// How long can we spend writing a response?
	WriteTimeout time.Duration

	// What is the maximum request length in bytes that we are
	// willing to handle?
	RequestMaxSize int64
}

type Conn interface {
	// This is a subset of net.Conn; semantics are the same.

	Read(b []byte) (n int, err error)
	Write(b []byte) (n int, err error)
	SetDeadline(t time.Time) error
	SetReadDeadline(t time.Time) error
	SetWriteDeadline(t time.Time) error
}

// Handle a request to nslcd
func HandleRequest(backend Backend, limits Limits, conn Conn, cred unix.Ucred) (err error) {
	defer func() {
		if r := recover(); r != nil {
			switch r := r.(type) {
			case p.NslcdError:
				err = r
			default:
				panic(r)
			}
		}
	}()

	now := time.Now()
	if limits.Timeout != 0 {
		err = conn.SetDeadline(now.Add(limits.Timeout))
		if err != nil {
			return err
		}
	}
	if limits.ReadTimeout != 0 {
		err = conn.SetReadDeadline(now.Add(limits.ReadTimeout))
		if err != nil {
			return err
		}
	}

	var in io.Reader = conn
	if limits.RequestMaxSize > 0 {
		in = &io.LimitedReader{R: in, N: limits.RequestMaxSize}
	}
	out := conn

	var version int32
	maybePanic(p.Read(in, &version))
	if version != p.NSLCD_VERSION {
		return p.NslcdError(fmt.Sprintf("Version mismatch: server=%#08x client=%#08x", p.NSLCD_VERSION, version))
	}
	var action int32
	maybePanic(p.Read(in, &action))

	switch action {
	case p.NSLCD_ACTION_CONFIG_GET:
		var req p.Request_Config_Get
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Config_Get(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_ALIAS_BYNAME:
		var req p.Request_Alias_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Alias_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_ALIAS_ALL:
		var req p.Request_Alias_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Alias_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_ETHER_BYNAME:
		var req p.Request_Ether_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Ether_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_ETHER_BYETHER:
		var req p.Request_Ether_ByEther
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Ether_ByEther(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_ETHER_ALL:
		var req p.Request_Ether_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Ether_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_GROUP_BYNAME:
		var req p.Request_Group_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Group_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_GROUP_BYGID:
		var req p.Request_Group_ByGid
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Group_ByGid(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_GROUP_BYMEMBER:
		var req p.Request_Group_ByMember
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Group_ByMember(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_GROUP_ALL:
		var req p.Request_Group_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Group_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_HOST_BYNAME:
		var req p.Request_Host_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Host_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_HOST_BYADDR:
		var req p.Request_Host_ByAddr
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Host_ByAddr(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_HOST_ALL:
		var req p.Request_Host_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Host_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_NETGROUP_BYNAME:
		var req p.Request_Netgroup_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Netgroup_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_NETGROUP_ALL:
		var req p.Request_Netgroup_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Netgroup_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_NETWORK_BYNAME:
		var req p.Request_Network_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Network_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_NETWORK_BYADDR:
		var req p.Request_Network_ByAddr
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Network_ByAddr(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_NETWORK_ALL:
		var req p.Request_Network_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Network_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PASSWD_BYNAME:
		var req p.Request_Passwd_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Passwd_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PASSWD_BYUID:
		var req p.Request_Passwd_ByUID
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Passwd_ByUID(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PASSWD_ALL:
		var req p.Request_Passwd_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Passwd_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PROTOCOL_BYNAME:
		var req p.Request_Protocol_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Protocol_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PROTOCOL_BYNUMBER:
		var req p.Request_Protocol_ByNumber
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Protocol_ByNumber(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PROTOCOL_ALL:
		var req p.Request_Protocol_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Protocol_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_RPC_BYNAME:
		var req p.Request_RPC_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.RPC_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_RPC_BYNUMBER:
		var req p.Request_RPC_ByNumber
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.RPC_ByNumber(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_RPC_ALL:
		var req p.Request_RPC_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.RPC_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_SERVICE_BYNAME:
		var req p.Request_Service_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Service_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_SERVICE_BYNUMBER:
		var req p.Request_Service_ByNumber
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Service_ByNumber(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_SERVICE_ALL:
		var req p.Request_Service_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Service_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_SHADOW_BYNAME:
		var req p.Request_Shadow_ByName
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Shadow_ByName(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_SHADOW_ALL:
		var req p.Request_Shadow_All
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.Shadow_All(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PAM_AUTHENTICATION:
		var req p.Request_PAM_Authentication
		maybePanic(p.Read(in, &req))
		_req := req
		_req.Password = sensitive
		fmt.Fprintf(os.Stderr, "Request: %#v\n", _req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.PAM_Authentication(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PAM_AUTHORIZATION:
		var req p.Request_PAM_Authorization
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.PAM_Authorization(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PAM_SESSIONOPEN:
		var req p.Request_PAM_SessionOpen
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.PAM_SessionOpen(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PAM_SESSIONCLOSE:
		var req p.Request_PAM_SessionClose
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.PAM_SessionClose(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_PAM_PWMOD:
		var req p.Request_PAM_PwMod
		maybePanic(p.Read(in, &req))
		_req := req
		if len(_req.OldPassword) > 0 {
			_req.OldPassword = sensitive
		}
		_req.NewPassword = sensitive
		fmt.Fprintf(os.Stderr, "Request: %#v\n", _req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.PAM_PwMod(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	case p.NSLCD_ACTION_USERMOD:
		var req p.Request_UserMod
		maybePanic(p.Read(in, &req))
		fmt.Fprintf(os.Stderr, "Request: %#v\n", req)
		if limits.WriteTimeout != 0 {
			err = out.SetWriteDeadline(time.Now().Add(limits.WriteTimeout))
			if err != nil {
				return err
			}
		}
		maybePanic(p.Write(out, p.NSLCD_VERSION))
		maybePanic(p.Write(out, action))
		ch := backend.UserMod(cred, req)
		for result := range ch {
			if err == nil {
				err = p.Write(out, p.NSLCD_RESULT_BEGIN)
			}
			if err == nil {
				err = p.Write(out, result)
			}
		}
		maybePanic(err)
		maybePanic(p.Write(out, p.NSLCD_RESULT_END))
		return nil
	default:
		return p.NslcdError(fmt.Sprintf("Unknown request action: %#08x", action))
	}
}
